@@ -15,9 +15,11 @@
 #include <assert.h>
 
 #include "wn.h"
+#ifdef WN1_6
 #include "setutil.h"
+#endif
 
-static char *Id = "$Id: search.c,v 1.136 1998/08/07 17:51:32 wn Exp $";
+static char *Id = "$Id: search.c,v 1.145 2001/06/19 15:01:22 wn Exp $";
 
 /* For adjectives, indicates synset type */
 
@@ -26,6 +28,8 @@ static char *Id = "$Id: search.c,v 1.136 1998/08/07 17:51:32 wn Exp $";
 #define INDIRECT_ANT	2	/* indrect antonyms (similar) */
 #define PERTAINYM	3	/* no antonyms or similars (pertainyms) */
 
+/* Flags for printsynset() */
+
 #define ALLWORDS	0	/* print all words */
 #define SKIP_ANTS	0	/* skip printing antonyms in printsynset() */
 #define PRINT_ANTS	1	/* print antonyms in printsynset() */
@@ -44,11 +48,14 @@ static char *Id = "$Id: search.c,v 1.136 1998/08/07 17:51:32 wn Exp $";
 /* Forward function declarations */
 
 static void WNOverview(char *, int);
-static void findsisters(IndexPtr), findtwins(IndexPtr), findcousins(IndexPtr),
-            findverbgroups(IndexPtr);
+#ifdef WN1_6
+static void findsisters(IndexPtr), findtwins(IndexPtr), findcousins(IndexPtr);
+static int read_cousintops(void);
+static int groupexc(unsigned long, unsigned long);
+#endif
+static void findverbgroups(IndexPtr);
 static void add_relatives(int, IndexPtr, int, int);
 static void free_rellist(void);
-static int read_cousintops(void);
 static void printsynset(char *, SynsetPtr, char *, int, int, int, int);
 static void printantsynset(SynsetPtr, char *, int, int);
 static char *printant(int, SynsetPtr, int, char *, char *);
@@ -62,8 +69,9 @@ static int HasHoloMero(IndexPtr, int);
 static int HasPtr(SynsetPtr, int);
 static int getsearchsense(SynsetPtr, int);
 static int depthcheck(int, SynsetPtr);
-static int groupexc(unsigned long, unsigned long);
 static void interface_doevents();
+static void getexample(char *, char *);
+static int findexample(SynsetPtr);
 
 /* Static variables */
 
@@ -71,21 +79,21 @@ static int prflag, sense, prlexid;
 static int overflag = 0;	/* set when output buffer overflows */
 static char searchbuffer[SEARCHBUF];
 static int lastholomero;	/* keep track of last holo/meronym printed */
-#define TMPBUFSIZE 1024*8
+#define TMPBUFSIZE 1024*10
 static char tmpbuf[TMPBUFSIZE];	/* general purpose printing buffer */
 static char wdbuf[WORDBUF];	/* general purpose word buffer */
 static char msgbuf[256];	/* buffer for constructing error messages */
 static int adj_marker;
 
+extern long last_bin_search_offset;
+
 
 /* Find word in index file and return parsed entry in data structure.
    Input word must be exact match of string in database. */
 
 IndexPtr index_lookup(char *word, int dbase)
 {
-    char *ptrtok;
     IndexPtr idx = NULL;
-    int j;
     FILE *fp;
     char *line;
 
@@ -97,72 +105,96 @@ IndexPtr index_lookup(char *word, int dbase)
     }
 
     if ((line = bin_search(word, fp)) != NULL) {
-        idx = (IndexPtr)malloc(sizeof(Index));
-        assert(idx);
-	
-        idx->wd='\0';
-        idx->pos='\0';
-        idx->off_cnt=0;
-	idx->tagged_cnt = 0;
-        idx->sense_cnt=0;
-        idx->offset='\0';
-        idx->ptruse_cnt=0;
-        idx->ptruse='\0';
-	
-	/* get the word */
-	ptrtok=strtok(line," \n");
-	
-	idx->wd = malloc(strlen(ptrtok) + 1);
-	assert(idx->wd);
-	strcpy(idx->wd, ptrtok);
-	
-	/* get the part of speech */
-	ptrtok=strtok(NULL," \n");
-	idx->pos = malloc(strlen(ptrtok) + 1);
-	assert(idx->pos);
-	strcpy(idx->pos, ptrtok);
-	
-	/* get the collins count */
-	ptrtok=strtok(NULL," \n");
-	idx->sense_cnt = atoi(ptrtok);
-	
-	/* get the number of pointers types */
-	ptrtok=strtok(NULL," \n");
-	idx->ptruse_cnt = atoi(ptrtok);
+	idx = parse_index( last_bin_search_offset, dbase, line);
+    } 
+    return (idx);
+}
 
-	if (idx->ptruse_cnt) {
-	    idx->ptruse = (int *) malloc(idx->ptruse_cnt * (sizeof(int)));
-	    assert(idx->ptruse);
-	
-	    /* get the pointers types */
-	    for(j=0;j < idx->ptruse_cnt; j++) {
-		ptrtok=strtok(NULL," \n");
-		idx->ptruse[j] = getptrtype(ptrtok);
-	    }
-	}
-	
-	/* get the number of offsets */
-	ptrtok=strtok(NULL," \n");
-	idx->off_cnt = atoi(ptrtok);
+/* This function parses an entry from an index file into an Index data
+ * structure. It takes the byte offset and file number, and optionally the
+ * line. If the line is NULL, parse_index will get the line from the file.
+ * If the line is non-NULL, parse_index won't look at the file, but it still
+ * needs the dbase and offset parameters to be set, so it can store them in
+ * the Index struct.
+ */
 
-	if (!strcmp(wnrelease, "1.6")) {
-	    /* get the number of senses that are tagged */
-	    ptrtok=strtok(NULL," \n");
-	    idx->tagged_cnt = atoi(ptrtok);
-	} else {
-	    idx->tagged_cnt = -1;
-	}
+IndexPtr parse_index(long offset, int dbase, char *line) {
+    
+    IndexPtr idx = NULL;
+    char *ptrtok;
+    int j;
+
+    if ( !line )
+      line = read_index( offset, indexfps[dbase] );
+    
+    idx = (IndexPtr)malloc(sizeof(Index));
+    assert(idx);
+
+    /* set offset of entry in index file */
+    idx->idxoffset = offset;
+    
+    idx->wd='\0';
+    idx->pos='\0';
+    idx->off_cnt=0;
+    idx->tagged_cnt = 0;
+    idx->sense_cnt=0;
+    idx->offset='\0';
+    idx->ptruse_cnt=0;
+    idx->ptruse='\0';
+    
+    /* get the word */
+    ptrtok=strtok(line," \n");
+    
+    idx->wd = malloc(strlen(ptrtok) + 1);
+    assert(idx->wd);
+    strcpy(idx->wd, ptrtok);
+    
+    /* get the part of speech */
+    ptrtok=strtok(NULL," \n");
+    idx->pos = malloc(strlen(ptrtok) + 1);
+    assert(idx->pos);
+    strcpy(idx->pos, ptrtok);
+    
+    /* get the collins count */
+    ptrtok=strtok(NULL," \n");
+    idx->sense_cnt = atoi(ptrtok);
+    
+    /* get the number of pointers types */
+    ptrtok=strtok(NULL," \n");
+    idx->ptruse_cnt = atoi(ptrtok);
+    
+    if (idx->ptruse_cnt) {
+	idx->ptruse = (int *) malloc(idx->ptruse_cnt * (sizeof(int)));
+	assert(idx->ptruse);
 	
-	/* make space for the offsets */
-	idx->offset = (long *) malloc(idx->off_cnt * (sizeof(long)));
-	assert(idx->offset);
-	
-	/* get the offsets */
-	for(j=0;j<idx->off_cnt;j++) {
+	/* get the pointers types */
+	for(j=0;j < idx->ptruse_cnt; j++) {
 	    ptrtok=strtok(NULL," \n");
-	    idx->offset[j] = atol(ptrtok);
+	    idx->ptruse[j] = getptrtype(ptrtok);
 	}
     }
+    
+    /* get the number of offsets */
+    ptrtok=strtok(NULL," \n");
+    idx->off_cnt = atoi(ptrtok);
+    
+    if (!strcmp(wnrelease, "1.6") || !strcmp(wnrelease, "1.7")) {
+	/* get the number of senses that are tagged */
+	ptrtok=strtok(NULL," \n");
+	idx->tagged_cnt = atoi(ptrtok);
+    } else {
+	idx->tagged_cnt = -1;
+    }
+    
+    /* make space for the offsets */
+    idx->offset = (long *) malloc(idx->off_cnt * (sizeof(long)));
+    assert(idx->offset);
+    
+    /* get the offsets */
+    for(j=0;j<idx->off_cnt;j++) {
+	ptrtok=strtok(NULL," \n");
+	idx->offset[j] = atol(ptrtok);
+    }
     return(idx);
 }
 
@@ -304,6 +336,7 @@ SynsetPtr parse_synset(FILE *fp, int dbase, char *word)
 	sprintf(msgbuf, "WordNet library error: no synset at location %d\n",
 		loc);
 	display_message(msgbuf);
+	free(synptr);
 	return(NULL);
     }
     
@@ -462,7 +495,6 @@ void free_syns(SynsetPtr synptr)
 
     if (synptr) {
 	cursyn = synptr;
-	nextsyn = synptr->nextss;
 	while(cursyn) {
 	    nextsyn = cursyn->nextss;
 	    free_synset(cursyn);
@@ -554,6 +586,10 @@ static void traceptrs(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
 		    sprintf(prefix, "Pertains to %s ",
 			    partnames[synptr->ppos[i]]);
 		break;
+	    case ANTPTR:
+		if (dbase != ADJ)
+		    sprintf(prefix, "Antonym of ");
+		break;
 	    case PPLPTR:
 		sprintf(prefix, "Participle of verb ");
 		break;
@@ -610,6 +646,13 @@ static void traceptrs(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
 				DEFON, ALLWORDS, PRINT_ANTS, PRINT_MARKER);
 		    traceptrs(cursyn, HYPERPTR, getpos(cursyn->pos), 0);
 		}
+	    } else if (ptrtyp == ANTPTR && dbase != ADJ && synptr->pto[i] != 0) {
+		sprintf(tbuf, " (Sense %d)\n",
+			cursyn->wnsns[synptr->pto[i] - 1]);
+		printsynset(prefix, cursyn, tbuf, DEFOFF, synptr->pto[i],
+			    SKIP_ANTS, PRINT_MARKER);
+		printsynset("      =>", cursyn, "\n", DEFON, ALLWORDS,
+			    PRINT_ANTS, PRINT_MARKER);
 	    } else 
 		printsynset(prefix, cursyn, "\n", DEFON, ALLWORDS,
 			    PRINT_ANTS, PRINT_MARKER);
@@ -668,6 +711,46 @@ static void tracecoords(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
     }
 }
 
+static void tracenomins(SynsetPtr synptr, int dbase)
+{
+    int i, j, idx;
+    SynsetPtr cursyn;
+    long int prlist[32];
+
+    interface_doevents();
+    if (abortsearch)
+	return;
+
+    idx = 0;
+
+    for (i = 0; i < synptr->ptrcount; i++) {
+	if ((synptr->ptrtyp[i] >= NOMIN_START) &&
+	    (synptr->ptrtyp[i] <= NOMIN_END)) {
+
+	    if (!prflag) {
+		printsns(synptr, sense + 1);
+		prflag = 1;
+	    }
+	    cursyn = read_synset(synptr->ppos[i], synptr->ptroff[i], "");
+
+	    for (j = 0; j < idx; j++) {
+		if (synptr->ptroff[i] == prlist[j]) {
+		    break;
+		}
+	    }
+
+	    if (j == idx) {
+		prlist[idx++] = synptr->ptroff[i];
+		printspaces(TRACEP, 0);
+		printsynset("<-> ", cursyn, "\n", DEFON, ALLWORDS,
+			    SKIP_ANTS, PRINT_MARKER);
+	    }
+
+	    free_synset(cursyn);
+	}
+    }
+}
+
 /* Trace through the hypernym tree and print all MEMBER, STUFF
    and PART info. */
 
@@ -735,7 +818,8 @@ static void partsall(SynsetPtr synptr, int ptrtyp)
     /* Print out MEMBER, STUFF, PART info for hypernyms on
        HMERONYM search only */
 	
-    if (hasptr && ptrtyp == HMERONYM) {
+/*    if (hasptr && ptrtyp == HMERONYM) { */
+    if (ptrtyp == HMERONYM) {
 	lastholomero = strlen(searchbuffer);
 	traceinherit(synptr, ptrbase, NOUN, 1);
     }
@@ -804,7 +888,8 @@ static void traceadjant(SynsetPtr synptr)
 }
 
 
-/* fetch the given example sentence from the example file and print it out */
+/* Fetch the given example sentence from the example file and print it out */
+
 void getexample(char *offset, char *wd)
 {
     char *line;
@@ -822,8 +907,9 @@ void getexample(char *offset, char *wd)
     }
 }
 
-/* find the example sentence references in the example sentence index file */
-int findExample(SynsetPtr synptr)
+/* Find the example sentence references in the example sentence index file */
+
+int findexample(SynsetPtr synptr)
 {
     char tbuf[256], *temp, *offset;
     int wdnum;
@@ -864,7 +950,7 @@ static void printframe(SynsetPtr synptr, int prsynset)
     if (prsynset)
 	printsns(synptr, sense + 1);
     
-    if (!findExample(synptr)) {
+    if (!findexample(synptr)) {
 	for(i = 0; i < synptr->fcount; i++) {
 	    if ((synptr->frmto[i] == synptr->whichword) ||
 		(synptr->frmto[i] == 0)) {
@@ -940,7 +1026,7 @@ static void freq_word(IndexPtr index)
 	if (cnt > 32 ) familiar = 7;
 	
 	sprintf(tmpbuf,
-		"%s used as %s is %s (polysemy count = %d)\n",
+		"\n%s used as %s is %s (polysemy count = %d)\n",
 		index->wd, a_an[getpos(index->pos)], freqcats[familiar], cnt);
 	printbuffer(tmpbuf);
     }
@@ -1008,9 +1094,13 @@ static struct relgrp *rellist;
 
 static struct relgrp *mkrellist(void);
 
+/* Simple hash function */
+#define HASHTABSIZE	1223	/* Prime number. Must be > 2*MAXTOPS */
+#define hash(n) ((n) % HASHTABSIZE)
+
+#ifdef WN1_6
 void trace_hyperptrs(SynsetPtr, void (*)(unsigned long, void *), void *, int);
 
-#define HASHTABSIZE	1223	/* Prime number. Must be > 2*MAXTOPS */
 #define MAXTOPS 	300	/* Maximum number of lines in cousin.tops */
 
 static struct {
@@ -1018,9 +1108,7 @@ static struct {
     Set_t rels;			/* set of top nodes this one is paired with */
     unsigned long offset;	/* Offset read from cousin.tops file */
 } cousintops[HASHTABSIZE];
-
-/* Simple hash function */
-#define hash(n) ((n) % HASHTABSIZE)
+#endif
 
 /* Find relative groups for all senses of target word in given part
    of speech. */
@@ -1031,6 +1119,8 @@ static void relatives(IndexPtr idx, int dbase)
 
     switch(dbase) {
     case NOUN:
+#ifdef WN1_6
+/* Don't bother for nouns - doesn't work well */
 	findsisters(idx);
 	interface_doevents();
 	if (abortsearch)
@@ -1044,6 +1134,7 @@ static void relatives(IndexPtr idx, int dbase)
 	if (abortsearch)
 	    break;
 	printrelatives(idx, NOUN);
+#endif
 	break;
     case VERB:
 	findverbgroups(idx);
@@ -1059,6 +1150,8 @@ static void relatives(IndexPtr idx, int dbase)
     free_rellist();
 }
 
+#ifdef WN1_6
+
 /* Look for 'twins' - synsets with 3 or more words in common. */
 
 static int word_idx(char *wd, char *wdtable[], int nwords)
@@ -1313,7 +1406,6 @@ static void findcousins(IndexPtr idx)
      }
      set_destroy(n);
 }
-
 /* Trace through HYPERPTRs up to MAXDEPTH, running `fn()' on each one. */ 
 
 void trace_hyperptrs(SynsetPtr synptr, 
@@ -1334,6 +1426,7 @@ void trace_hyperptrs(SynsetPtr synptr,
 	       free_synset(s);
 	  }
 }
+#endif
 
 static void findverbgroups(IndexPtr idx)
 {
@@ -1371,9 +1464,11 @@ static void add_relatives(int pos, IndexPtr idx, int rel1, int rel2)
     int i;
     struct relgrp *rel, *last, *r;
 
+#ifdef WN1_6
     /* First make sure that senses are not on the excpetion list */
     if (pos == NOUN && groupexc(idx->offset[rel1], idx->offset[rel2]))
 	return;
+#endif
 
     /* If either of the new relatives are already in a relative group,
        then add the other to the existing group (transitivity).
@@ -1402,10 +1497,11 @@ static void add_relatives(int pos, IndexPtr idx, int rel1, int rel2)
     else
 	last->next = rel;
 }
-    
+
+#ifdef WN1_6
 static int groupexc(unsigned long off1, unsigned long off2)
 {
-    char buf[8], *p, linebuf[1024];
+    char buf[9], *p, linebuf[1024];
 
     sprintf(buf, "%8.8lu", (off1 < off2 ? off1 : off2));
 
@@ -1419,6 +1515,7 @@ static int groupexc(unsigned long off1, unsigned long off2)
     }
     return(p ? 1 : 0);
 }
+#endif
 
 static struct relgrp *mkrellist(void)
 {
@@ -1529,11 +1626,13 @@ char *findtheinfo(char *searchstr, int dbase, int ptrtyp, int whichsense)
     case WNGREP:
 	wngrep(searchstr, dbase);
 	break;
+#ifndef WN1_7
     case WNESCORT:
 	sprintf(searchbuffer,
 	"Sentences containing %s will be displayed in the Escort window.",
 		searchstr);
 	break;
+#endif
     case RELATIVES:
     case VERBGROUP:
 	while ((idx = getindex(searchstr, dbase)) != NULL) {
@@ -1652,7 +1751,11 @@ char *findtheinfo(char *searchstr, int dbase, int ptrtyp, int whichsense)
 			    if (dbase == VERB && frflag)
 			    	printframe(cursyn, 0);
 			    break;
-		    
+
+			case NOMINALIZATIONS:
+			    tracenomins(cursyn, dbase);
+			    break;
+
 		    	default:
 			    traceptrs(cursyn, ptrtyp, dbase, depth);
 			    break;
@@ -1887,7 +1990,13 @@ static void WNOverview(char *searchstr, int pos)
 	    if (!skipit) {
 		offsets[offsetcnt++] = idx->offset[sense];
 		cursyn = read_synset(pos, idx->offset[sense], idx->wd);
-		sprintf(tmpbuf, "%d. ", sense + 1);
+		if (idx->tagged_cnt != -1 &&
+		    ((sense + 1) <= idx->tagged_cnt)) {
+		  sprintf(tmpbuf, "%d. (%d) ",
+			  sense + 1, GetTagcnt(idx, sense + 1));
+		} else {
+		  sprintf(tmpbuf, "%d. ", sense + 1);
+		}
 
 		svdflag = dflag;
 		dflag = 1;
@@ -1941,7 +2050,7 @@ char *do_trace(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
 /* Set bit for each search type that is valid for the search word
    passed and return bit mask. */
   
-unsigned long is_defined(char *searchstr, int dbase)
+unsigned int is_defined(char *searchstr, int dbase)
 {
     IndexPtr index;
     int i;
@@ -1961,35 +2070,57 @@ unsigned long is_defined(char *searchstr, int dbase)
 	retval |= bit(SIMPTR) | bit(FREQ) | bit(SYNS)|
 	    bit(WNGREP) | bit(OVERVIEW);
 
+	/* go through list of pointer characters and set appropriate bits */
+
 	for(i = 0; i < index->ptruse_cnt; i++) {
-	    
-	    retval |= bit(index->ptruse[i]); /* set bit for this ptr */
 
-	    if (dbase == NOUN) {
+	    if (index->ptruse[i] <= LASTTYPE) {
+		retval |= bit(index->ptruse[i]);
+	    }
 
-		/* set generic HOLONYM and/or MERONYM bit if necessary */
-	    
-		if(index->ptruse[i] >= ISMEMBERPTR &&
-		   index->ptruse[i] <= ISPARTPTR)
-		    retval |= bit(HOLONYM);
-		else if(index->ptruse[i] >= HASMEMBERPTR &&
-			index->ptruse[i] <= HASPARTPTR)
-		    retval |= bit(MERONYM);
-	    } else if (dbase == ADJ && index->ptruse[i] == SIMPTR)
+	    if (index->ptruse[i] >= NOMIN_START &&
+		 index->ptruse[i] <= NOMIN_END) {
+		retval |= bit(NOMINALIZATIONS);
+	    }
+
+	    if (index->ptruse[i] >= ISMEMBERPTR &&
+	       index->ptruse[i] <= ISPARTPTR)
+		retval |= bit(HOLONYM);
+	    else if (index->ptruse[i] >= HASMEMBERPTR &&
+		    index->ptruse[i] <= HASPARTPTR)
+		retval |= bit(MERONYM);
+	 
+	    if (index->ptruse[i] == SIMPTR) {
 		retval |= bit(ANTPTR);
+	    } 
 	}
 
 	if (dbase == NOUN) {
+#ifdef WN1_6
 	    retval |= bit(RELATIVES);
+#endif
+	    /* check for inherited holonyms and meronyms */
+
 	    if (HasHoloMero(index, HMERONYM))
 		retval |= bit(HMERONYM);
 	    if (HasHoloMero(index, HHOLONYM))
 		retval |= bit(HHOLONYM);
+
+	    /* if synset has hypernyms, enable coordinate search */
+
 	    if (retval & bit(HYPERPTR))
 		retval |= bit(COORDS);
 	} else if (dbase == VERB) {
-	    retval |= bit(RELATIVES);
+
+	    /* if synset has hypernyms, enable coordinate search */
+	    if (retval & bit(HYPERPTR))
+		retval |= bit(COORDS);
+
+	    /* enable grouping of related synsets and verb frames */
+
+	    retval |= bit(RELATIVES) | bit(FRAMES);
 	}
+
 	free_index(index);
 	wnresults.numforms++;
     }
@@ -2001,18 +2132,25 @@ unsigned long is_defined(char *searchstr, int dbase)
 
 static int HasHoloMero(IndexPtr index, int ptrtyp)
 {
-    int i;
-    SynsetPtr synset;
+    int i, j;
+    SynsetPtr synset, psynset;
     int found=0;
     int ptrbase;
-    
+
     ptrbase = (ptrtyp == HMERONYM) ? HASMEMBERPTR : ISMEMBERPTR;
     
     for(i = 0; i < index->off_cnt; i++) {
 	synset = read_synset(NOUN, index->offset[i], "");
-	found += HasPtr(synset, ptrbase);
-	found += HasPtr(synset, ptrbase + 1);
-	found += HasPtr(synset, ptrbase + 2);
+	for (j = 0; j < synset->ptrcount; j++) {
+	    if (synset->ptrtyp[j] == HYPERPTR) {
+		psynset = read_synset(NOUN, synset->ptroff[j], "");
+		found += HasPtr(psynset, ptrbase);
+		found += HasPtr(psynset, ptrbase + 1);
+		found += HasPtr(psynset, ptrbase + 2);
+
+		free_synset(psynset);
+	    }
+	}
 	free_synset(synset);
     }
     return(found);
@@ -2320,7 +2458,7 @@ static void printspaces(int trace, int depth)
 	printbuffer("    ");
 
     switch(trace) {
-    case TRACEP:		/* traceptrs() */
+    case TRACEP:		/* traceptrs(), tracenomins() */
 	if (depth)
 	    printbuffer("   ");
 	else
@@ -2350,6 +2488,35 @@ static void interface_doevents (void) {
   Revision log: (since version 1.5)
   
   $Log: search.c,v $
+  Revision 1.145  2001/06/19 15:01:22  wn
+  commed out include for setutil.h
+
+  Revision 1.144  2001/05/30 16:24:17  wn
+  changed is_defined to return unsigned int
+
+  Revision 1.143  2001/03/30 17:13:00  wn
+  fixed is_defined - wasn't setting coords for verbs
+
+  Revision 1.142  2001/03/29 16:18:03  wn
+  added newline before output from FREQ search
+
+  Revision 1.141  2001/03/29 16:11:39  wn
+  added code to tractptrs to print direct antonyms nicer
+
+  Revision 1.140  2001/03/27 18:47:41  wn
+  removed tcflag
+
+  Revision 1.139  2001/03/27 16:47:44  wn
+  updated is_defined for holonyms and meronyms
+
+  Revision 1.138  2000/08/14 16:04:24  wn
+  changed 'get_index' to call sub to do work
+  added code for nominalizations
+
+  Revision 1.137  1998/08/11 18:07:11  wn
+  minor fixes: free synptr space before rreturning if error; remove
+  useless statement in free_syns
+
  * Revision 1.136  1998/08/07  17:51:32  wn
  * added COORDS to traceptrs_ds and findtheinfo_ds
  * fixed getsearchsense code to only happen in parse_synset
